"""cubicweb-postgis application package

Wrap Postgis into CubicWeb.
"""

_GEOM_PARAMETERS = ('srid', 'geom_type', 'coord_dimension')  # XXX: coord_dimension -> z and m ?
_GEOG_PARAMETERS = ('srid', 'geom_type', 'coord_dimension')
